const { Client } = require("@elastic/elasticsearch");
import { O_DIRECT } from "constants";
import {
  convertCompilerOptionsFromJson,
  createNoSubstitutionTemplateLiteral,
} from "typescript";
import { HttpService } from "./http.service";

export class ElasticSearchService {
  client: any;
  httpService: HttpService;
  index: string;
  constructor(host: string = "http://localhost:9200", index = "my-index") {
    this.client = new Client({ node: host });
    this.httpService = new HttpService(host);
    this.index = index;
  }

  async addItem(obj: any) {
    try {
      const ans = await this.httpService.addItem(this.index, obj, obj.id);
      this.refreshIndex();
      return ans;
    } catch (error) {
      console.log("add Item error: ", error);
    }
  }
  // async addItem(index: string, obj: any) {
  //   try{

  //     const { body } = await this.client.index({
  //       index: index,
  //       body: obj,
  //     });
  //     this.refreshIndex(index);
  //     return body._id;
  //   }catch(error){
  //     console.log('add Item error: ',error)
  //   }
  // }
  async updateItem(obj: any) {
    if (!obj?.id) {
      return false;
    }
    const ans = await this.httpService.updateItem(this.index, obj.id, obj);
    console.log("ans: ", ans);
    return ans;
  }

  async addMultipleItems(objArray: Array<any>) {
    console.log("123");
    var idsArr = [];
    for (const obj of objArray) {
      const id = await this.addItem(obj);
      idsArr.push(id);
    }
    return idsArr;
  }

  async deleteDocumentByElasticId(id: string) {
    const ans = await this.httpService.deleteByElasticIp(this.index, id);
    console.log("ans: ", ans);
    return ans;
  }
  async deleteByField(index: string, field: string, value: string) {
    const ans = await this.httpService.deleteByField(index, field, value);
    console.log("ans: ", ans);
    return ans;
  }
  async deleteByMultipleFields(deleteObj: any) {
    const query = await this.buildQuery(deleteObj);
    const ans = await this.httpService.deleteByMultipleFields(
      this.index,
      query
    );
    console.log("ans: ", ans);
    return ans;
  }

  async searchByField(
    field: string,
    value: string,
    limit: any = null,
    offset: any = null
  ) {
    const objToSend: any = {
      index: this.index,
      body: {
        query: {
          match: { [field]: value },
        },
      },
    };
    if (offset) {
      objToSend.from = offset;
    }
    if (limit) {
      objToSend.size = limit;
    }
    const { body } = await this.client.search(objToSend);
    const result = await this.desrilize(body.hits.hits);
    return result;
  }
  async search(searchObj: any, limit: any = null, offset: any = null) {
    const query = await this.buildQuery(searchObj);
    const objToSend: any = {
      index: this.index,
      body: {
        query: {
          bool: {
            must: query,
          },
        },
      },
    };
    if (offset) {
      objToSend.from = offset;
    }
    if (limit) {
      objToSend.size = limit;
    }
    const { body } = await this.client.search(objToSend);
    const result = await this.desrilize(body.hits.hits);
    return result;
  }
  async getByElasticId(id: string) {
    // const query = await this.buildQuery(searchObj);
    const result = await this.httpService.getByElasticId(this.index, id);
    const result2 = await this.desrilize(result);
    return result2[0];
    // return result;
  }
  async searchByOneOfMultiFields(
    fields: Array<string>,
    txt: string,
    limit: any = null,
    offset: any = null
  ) {
    const result: any = await this.httpService.searchByOneOfMultiFields(
      this.index,
      fields,
      txt,
      limit,
      offset
    );
    if (result?.result?.hits) {
      const result2 = await this.desrilize(result.result.hits);
      return result2;
    } else {
      return result;
    }
  }

  async refreshIndex() {
    await this.client.indices.refresh({ index: this.index });
  }
  async buildQuery(obj: any) {
    var query = [];
    for (const key in obj) {
      query.push({
        match: {
          [key]: obj[key],
        },
      });
    }
    console.log("query: ", query);
    return query;
  }

  async desrilize(obj: any) {
    if (!Array.isArray(obj)) {
      obj = [obj];
    }
    const newObj = obj.map((obj: any) => {
      return obj._source;
    });
    return newObj;
  }
}
